<?php

namespace Myriad\Installations;

use Illuminate\Contracts\Foundation\Application;
use Illuminate\Support\ServiceProvider;
use Myriad\Installations\Validation\InstallationValidationProvider;
use Myriad\Installations\Validation\InstallationValidationProviderContract;

/**
 * Makes Installations available to Laravel applications.
 */
class InstallationsServiceProvider extends ServiceProvider
{

    /**
     * Perform post-registration booting of services.
     *
     * @return void
     */
    public function boot(): void
    {
        // Include routes
        if (!$this->app->routesAreCached() && config('installations.routes.enabled')) {
            require __DIR__.'/routes.php';
        }

        // Publish config and migrations
        $this->publishes([
          __DIR__.'/../database/migrations/' => database_path('migrations'),
        ], 'migrations');

        $this->publishes([
          __DIR__.'/../config/installations.php' => config_path('installations.php'),
        ], 'config');
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register(): void
    {
        $this->mergeConfigFrom(__DIR__.'/../config/installations.php', 'installations');

        // register the validator
        $this->app->singleton('installations.validator',
          config('installations.validator', InstallationValidationProvider::class)
        );

        $this->app->alias('installations.validator', InstallationValidationProviderContract::class);

        // register the manager
        $this->app->singleton('installations.manager', function (Application $app) {
            $request = $app->make('request');
            $config = $app->make('config');
            $validator = $app->make('installations.validator');
            return new InstallationManager($request, $config, $validator);
        });

        $this->app->alias('installations.manager', InstallationManager::class);
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides(): array
    {
        return [
          'installations.manager',
          InstallationManager::class,
          'installations.validator',
          InstallationValidationProvider::class,
          InstallationValidationProviderContract::class,
        ];
    }

}
