<?php

namespace Myriad\Installations\Middleware;

use Closure;
use Illuminate\Contracts\Validation\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Myriad\Installations\Exceptions\InstallationValidationException;
use Myriad\Installations\InstallationManager;

/**
 * Middleware for ensuring the installation data is passed correctly and saved consistently.
 */
class VerifyInstallation
{

    /**
     * @var InstallationManager knows how to operate on installations.
     */
    private $manager;

    /**
     * @var Factory knows how to create validators.
     */
    private $validator;

    /**
     * Create a new middleware instance.
     *
     * @param  InstallationManager  $manager  knows how to operate on installations.
     * @param  Factory  $validator  knows how to create validators.
     */
    public function __construct(InstallationManager $manager, Factory $validator)
    {
        $this->manager = $manager;
        $this->validator = $validator;
    }

    /**
     * Handle an incoming request.
     *
     * @param  Request  $request  Knows how to access the contents of the HTTP request.
     * @param  Closure  $next  The next middleware to run.
     *
     * @return mixed
     * @throws InstallationValidationException
     */
    public function handle($request, Closure $next)
    {
        $this->manager->setRequest($request);
        $params = $this->manager->getParamsFromRequest();

        $rules = Arr::only($this->manager->getValidator()->getRules(), array_keys($params));
        $messages = Arr::only($this->manager->getValidator()->getMessages(), array_keys($params));

        $validator = $this->validator->make($params, $rules, $messages);

        if ($validator->fails()) {
            throw new InstallationValidationException($validator);
        }

        $this->manager->createOrUpdateInstallation($this->manager->getUuidFromRequest(), $params);

        return $next($request);
    }

}
