<?php

namespace Myriad\Installations\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Installation extends Model
{

    use SoftDeletes;

    /**
     * The attributes that aren't mass assignable.
     *
     * @var array
     */
    protected $guarded = [
      'id',
      'uuid',
      'custom',
      'created_at',
      'updated_at',
      'deleted_at',
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
      'id',
      'custom',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
      'custom' => 'json',
    ];

    /**
     * Get the table associated with the model.
     *
     * @return string
     */
    public function getTable()
    {
        return config('installations.table_name', 'installations');
    }

    /**
     * Put a key value pair to custom.
     *
     * @param  string  $key  The key of the custom value.
     * @param  string  $value  The value associated with the key.
     */
    public function putCustom($key, $value)
    {
        $this->custom = array_merge(
          is_array($this->custom) ?
            $this->custom
            : [], [$key => $value]
        );
    }

    /**
     * Get a value from custom.
     *
     * @param  string  $key  The key to get.
     * @param  mixed  $default  The value to return if the key doesn't exist.
     * @return mixed
     */
    public function getCustom($key, $default)
    {
        $custom = $this->custom;

        if (!is_array($custom)) {
            return $default;
        }

        return array_key_exists($key, $custom) ? $custom[$key] : $default;
    }

}
